#!/usr/bin/python
#-----------------------------------------------------------------------
# File    : diags.py
# Contents: create detection diagrams
# Author  : Christian Borgelt
# History : 2012.??.?? file created
#           2014.05.27 modules placed in sibling directories
#-----------------------------------------------------------------------
from sys     import argv, path
from os.path import join
from math    import log10
UTILDIR = join('..', 'util')
if UTILDIR not in path: path.insert(0, UTILDIR)
from id2par  import get_args

#-----------------------------------------------------------------------

def header (out, pro):
    out.write('verbatimtex\n')
    out.write('\\documentclass[12pt]{article}\n')
    out.write('\\usepackage{mathptmx}\n')
    out.write('\\def\\hnorm{\\hbox to0pt{\\phantom{g)}\\hss}}\n')
    out.write('\\begin{document}\n')
    out.write('etex\n\n')
    out.write('prologues := ' +str(pro) +';\n\n')
    out.write('input axes;\n')
    out.write('input barchart;\n\n')
    out.write('picture Accretion; '
             +'Accretion = btex Accretion\\hnorm     etex;\n')
    out.write('picture None;      '
             +'None      = btex None\\hnorm          etex;\n')
    out.write('picture Weak;      '
             +'Weak      = btex Weak~2\\hnorm        etex;\n')
    out.write('picture WeakX;     '
             +'WeakX     = btex Weak~3\\hnorm        etex;\n')
    out.write('picture Strong;    '
             +'Strong    = btex Strong~2\\hnorm      etex;\n')
    out.write('picture StrongX;   '
             +'StrongX   = btex Strong~3\\hnorm      etex;\n\n')
    out.write('picture Chisqr;    '
             +'Chisqr    = btex $\chi^2$-test\\hnorm etex;\n')
    out.write('picture Yates;     '
             +'Yates     = btex Yates\\hnorm         etex;\n')
    out.write('picture Gtest;     '
             +'Gtest     = btex $G$-test\\hnorm      etex;\n')
    out.write('picture Fisher;    '
             +'Fisher    = btex Fisher\\hnorm        etex;\n')
    out.write('picture NoTest;    '
             +'NoTest    = btex no test\\hnorm       etex;\n\n')
    out.write('picture PatSize;   '
             +'PatSize   = btex \\small pattern size~$z$\\hnorm etex;\n')
    out.write('picture AssSize;   '
             +'AssSize   = btex \\small assembly size~$z$\\hnorm etex;\n')
    out.write('picture Coins;     '
             +'Coins     = btex \\small coincidences~$c$\\hnorm etex;\n')
    out.write('picture LogPats;   '
             +'LogPats   = btex \\small '
             +'$\\log(\\mbox{\\#patterns})$\\hnorm etex;\n')
    out.write('picture Rate;      '
             +'Rate      = btex \\small rate\\hnorm etex;\n\n')
    out.write('numeric expand;    expand    = 1/3;\n')
    out.write('numeric e;         e         = 2.718281828;\n\n')
    out.write('vardef init (expr cat, ttop, tbot) =\n')
    out.write('  numeric data[][];\n')
    out.write('  numeric zmin, zmax, n; n = 0;\n')
    out.write('  if cat = "fp": zmin = -3; zmax = 3;\n')
    out.write('  else:          zmin =  0; zmax = 1; fi\n')
    out.write('  barchart.set_defaults("ptmr8r");\n')
    out.write('  barchart.clipcolor := 0.7white;\n')
    out.write('  barchart.set_units(30mm/8, 30mm/8,')
    out.write(                    ' 24mm/(zmax-zmin));\n')
    out.write('  barchart.set_ranges((8,0.84,0.08), (8,0.84,0.08),'
                                  +' (zmin,zmax));\n')
    out.write('  barchart.set_view (-120, 36);\n')
    out.write('  barchart.set_light(-120, 75);\n')
    out.write('  barchart.null.max     := zmin-0.5;\n')
    out.write('  barchart.invalid.max  := zmin-1.5;\n')
    out.write('  axes.title.size       := 1.2;  % 12pt\n')
    out.write('  axes.title.skip       := 1.2 *fontsize "ptmr8r";\n')
    out.write('  axes.x.tlabel.size    := 8/10; %  8pt\n')
    out.write('  axes.x.alabel.size    := 1;    % 10pt\n')
    out.write('  axes.x.alabel.offset  := (0,4mm);\n')
    out.write('  axes.y.tlabel.size    := 8/10; %  8pt\n')
    out.write('  axes.y.alabel.size    := 1;    % 10pt\n')
    out.write('  axes.y.alabel.offset  := (0,4mm);\n')
    out.write('  axes.z.tlabel.size    := 8/10; %  8pt\n')
    out.write('  axes.z.alabel.size    := 1;    % 10pt\n')
    out.write('  axes.z.grid.thickness := 0.4pt;\n')
    out.write('  axes.z.kinked         := true;\n')
    out.write('  if cat = "fp":\n')
    out.write('    axes.z.alabel.offset := (1.0mm,5.5mm);\n')
    out.write('    barchart.draw_xlabel(0, PatSize);\n')
    out.write('    barchart.draw_ylabel(0, Coins);\n')
    out.write('    barchart.draw_zlabel(0, LogPats);\n')
    out.write('    barchart.draw_zticks(-3,-2,-1,0,1,2,3);\n')
    out.write('  else:\n')
    out.write('    axes.z.alabel.offset := (1.0mm,6mm);\n')
    out.write('    barchart.draw_xlabel(0, AssSize);\n')
    out.write('    barchart.draw_ylabel(0, Coins);\n')
    out.write('    barchart.draw_zlabel(0, Rate);\n')
    out.write('    barchart.draw_zticks(0,0.2,0.4,0.6,0.8,1);\n')
    out.write('  fi\n')
    out.write('  barchart.draw_xticks("2", "3", "4", "5",'
                                   +' "6", "7", "8", "9");\n')
    out.write('  barchart.draw_yticks("2", "3", "4", "5",'
                                   +' "6", "7", "8", "9");\n')
    out.write('  barchart.draw_title(-3, -1, ttop, tbot);\n')
    out.write('  barchart.draw_frame;\n')
    out.write('enddef;\n\n')
    out.write('vardef set_data (text list) =\n')
    out.write('  barchart.set_data(data[n], list);\n')
    out.write('  n := n+1;\n')
    out.write('enddef;\n\n')

#-----------------------------------------------------------------------

def footer (out):
    out.write('end.\n')

#-----------------------------------------------------------------------

def readfile (fname):
    with open(join('data', fname), 'r') as inp:
        c = [l.split() for l in inp]
    return dict([((int(x),int(y)),float(z)) for x,y,z in c])

#-----------------------------------------------------------------------

def diag (out, fid, cat, args):
    algo = args['algo']; stat = args['stat']
    name = cat+'_%s_%s_n%d_p%g_r%g_w%g_%c' \
         % (algo,stat,args['ncnt'],args['period'],args['rate'],
            args['width']*1000,args['target'])
    if   algo == 'acc': top = 'Accretion'
    elif algo == 'apr': top = 'None'
    elif algo == 'a-2': top = 'Weak'
    elif algo == 'a-3': top = 'WeakX'
    elif algo == 'a+2': top = 'Strong'
    elif algo == 'a+3': top = 'StrongX'
    else:               top = '""'
    if   stat == 'p':   bot = 'Chisqr'
    elif stat == 't':   bot = 'Yates'
    elif stat == 'g':   bot = 'Gtest'
    elif stat == 'f':   bot = 'Fisher'
    else:               bot = 'NoTest'
    vals = readfile(name+'.txt')
    out.write('beginfig(' +str(fid) +'); % ' +name +'\n')
    out.write('  init("'+cat+'", '+top+', '+bot+');\n')
    for s in range(2,10):       # traverse the group sizes (neurons)
        out.write('  set_data(')# start the next data row
        for c in range(2,10):   # traverse the numbers of coincidences
            if c > 2: out.write(', ')
            z = vals[(s,c)] if (s,c) in vals else 0
            if cat == 'fp':
                z = log10(z) if z > 0 else -4
            else:
                if cat == 'xs': z = 1-z
                if z <= 0: z = -1
            out.write('%f' % (z if z <= 3 else 4))
        out.write(');\n')       # print the values/fill data matrix
    out.write('  barchart.draw_data(data, 0, 5, expand);\n')
    out.write('endfig;\n\n')    # draw the diagram

#-----------------------------------------------------------------------

if __name__ == '__main__':
    grids = [ [ ( 'ncnt',   [  100 ] ),
                ( 'period', [    3 ] ),
                ( 'rate',   [   20 ] ),
                ( 'target', [ 'm', 'c' ] ),
                ( 'width',  [ 0.003, 0.004, 0.005 ] ),
                ( 'algo',   [ 'apr', 'a-2', 'a+2', 'a-3', 'a+3' ] ),
                ( 'stat',   [ 'p', 't', 'g', 'f' ] ) ],
              [ ( 'ncnt',   [  100 ] ),
                ( 'period', [    3 ] ),
                ( 'rate',   [   20 ] ),
                ( 'target', [ 'm', 'c' ] ),
                ( 'width',  [ 0.003, 0.004, 0.005 ] ),
                ( 'algo',   [ 'apr' ] ),
                ( 'stat',   [ 'x' ] ) ],
              [ ( 'ncnt',   [  100 ] ),
                ( 'period', [    3 ] ),
                ( 'rate',   [   20 ] ),
                ( 'target', [ 'm' ]),
                ( 'width',  [ 0.003, 0.004, 0.005 ] ),
                ( 'algo',   [ 'acc' ] ),
                ( 'stat',   [ 'p', 't', 'g', 'f' ] ) ],
              [ ( 'ncnt',   [  100 ] ),
                ( 'period', [    3 ] ),
                ( 'rate',   [ 30, 40 ] ),
                ( 'target', [ 'm' ]),
                ( 'width',  [ 0.003, 0.004, 0.005 ] ),
                ( 'algo',   [ 'a+3' ] ),
                ( 'stat',   [ 'p' ] ) ] ]
    fid = 0
    with open(argv[1] if len(argv) > 1 else 'accfim.mp', 'w') as out:
        header(out, 2)
        for i in range(get_args(grids)):
            args = get_args(grids, i)
            if args['rate'] <= 20:
                diag(out, fid, 'fp', args); fid += 1
            diag(out, fid, 'fn', args); fid += 1
            diag(out, fid, 'xs', args); fid += 1
        footer(out)
